/* Program GFSH, Version 1.0.3
*  Authors: Jochen Wolf, Marie Wolfova 2011
*/

#include <stdio.h>
#include <math.h>
#include <time.h>
#define ACM 80 /* Maximal dimension of the matrix for gene flow + 1 */
#define CB 3 /* Differentiates between crossbreeding and purebreeding
                Index 1: pure-bred animals, 2: cross-bred animals,
		0: sum of pure-bred and cross-bred animals */  
#define L 16 /* Maximal number of reproductive cycles + 1 */
#define NT 36 /* Maximal number of traits + 1 */
#define R 6 /* Maximal number of breeding cycles for rams + 1 */

void outputrw(FILE *fr, FILE *fw);
void outputr(FILE *fr);
void sysvar(FILE *fr, FILE *fw);
void readcom(FILE *fr);
float readdat(FILE *fr);
int readdat1(FILE *fr);
int readsyva(FILE *fr);
float readfloat(FILE *fw, FILE *fr);
int readint(FILE *fw, FILE *fr);
void readdes(FILE *fr);
void inp00(FILE *fw, int n2, int n3, int zz);
void wrides(FILE *fr, FILE *fw);
void nonsense01(int i, int prods);
void nonsense02(int acm, int prods);

clock_t zac, kon;

main()
{  
  FILE *fr, *fr2, *fw, *fw2;
  char out [128];
  int ac,acdA,acdB,acm,acsA,acsAB,acsB,delta_d[NT],delta_m[NT],
    errmess,flag[7],gstd,i,i1,j,l_inv,LLA,LLB,n_sac,
    n2,n3,pgdsA,pgdsB,pgssA,pgssB,prods,RRA,RRB,tt[NT],zz;
  float a2,a3,ageE1A,ageE1B,ageR11A,ageR11B,ageR12A,evpA[NT][CB],evpB[NT][CB],
    ew_d[NT],ew_m[NT],ewr_da[NT],ewr_dd[NT],ewr_ma[NT],ewr_mm[NT],
    ewst_d[NT],ewst_m[NT],ewsum,ewsum_d,ewsum_m,gstd_d[NT],gstd_m[NT],
    h_d[ACM],h_m[ACM],he_d[NT][ACM],he_m[NT][ACM],m_gf[ACM],m_gft[ACM],m_sum[ACM],
    P[ACM][ACM],pcrossE[L],pdA,pdB,u;
  double l4EA[L],l4EB[L],l4RA[R],l4RB[R];  
    
  zac = clock(); /* Start of measuring time for calculation */

/* Starting values */
  for (i = 0; i <= 6; i++) flag[i] = 0;
  for (i = 0; i < L; i++)
    l4EA[i] = l4EB[i] = pcrossE[i] = 0;
  for (i = 0; i < R; i++) 
    l4RA[i] = l4RB[i] = 0;
  for (i = 0; i < NT; i++) {  
    tt[i] = delta_d[i] = delta_m[i] = 0;  
    ew_d[i] = ew_m[i] = ewr_da[i] = ewr_dd[i] = ewr_ma[i] = ewr_mm[i] =
      ewst_d[i] = ewst_m[i] = gstd_d[i] = gstd_m[i] = 0.000000;
    for (j = 0; j < CB; j++) evpA[i][j] = evpB[i][j] = 0; 
    for (j = 0; j < ACM; j++) he_d[i][j] = he_m[i][j] = 0;
  }
  for (i = 0; i < ACM; i++)
    h_d[i] = h_m[i] = m_gf[i] = m_gft[i] = m_sum[i] = 0;
  for (i = 0; i < ACM; i++) {    
    for (j = 0; j < ACM; j++) P[i][j] = 0;   
  }
  
  ac = acdA = acdB = acm = acsA = acsAB = acsB = errmess = i = j = l_inv = LLA = 
    LLB = n_sac = n2 = pgdsA = pgdsB = pgssA = pgssB = prods = RRA = RRB = zz = 
    0;
  a2 = a3 = ageE1A = ageE1B = ageR11A = ageR11B = ageR12A = ewsum = ewsum_d = ewsum_m =
    pdA = pdB = u = 0.0;

/* Type of traits: 0 - direct, 1 - maternal, 2 - direct and maternal */
  for (i = 1; i <= 3; i++) tt[i] = 2;
  tt[21] = 1;
  tt[8] = tt[22] = 1;
  for (i = 23; i <= 24; i++) tt[i] = 2;
  for (i = 26; i <= 35; i++) tt[i] = 1;
  for (i = 1; i <= NT; i++) {
    if (tt[i] == 0 || tt[i] == 2) delta_d[i] = 1;
    if (tt[i] == 1 || tt[i] == 2) delta_m[i] = 1;
  }
         
Part_01:  
  printf("\nRunning GFSH Version 1.0.3, 2011\n"); 
  printf("Authors: Jochen Wolf (wolf.jochen@vuzv.cz)\n");
  printf("         Marie Wolfova (wolfova.marie@vuzv.cz)\n");
  /* Reading the file ERROR.TXT */
  fr = fopen("ERROR.TXT", "r");
  fscanf(fr, "%d", &errmess);
  close(fr);
  if (errmess == 1) {
    printf("\nError message:\n");
    printf("Program was stopped because there was an error message in one of the\n"); 
    printf("  preceding runs of EWSH2.\n\n");    
    goto END;  
  }
  /* Writing the head of the output file */
  fr2 = fopen("TEXTGFS_OUT.TXT", "r"); /* File for reading text for output */
  fr = fopen("P01.TXT", "r"); /* File for reading # of production system */
  fscanf(fr, "%d", &prods);
  close(fr);
  if (prods == 1) fw = fopen("resgfps01", "w"); /* File for writing results */  
  if (prods == 2) fw = fopen("resgfps02", "w");   
  outputrw(fr2, fw);
  fprintf(fw, "%3d", prods);
  outputrw(fr2, fw);  
  /* Reading parameter file PARAGFS.TXT and writing its information to output file */
  fr = fopen("PARAGFS.TXT", "r");
  outputrw(fr, fw);
  sysvar(fr, fw);
  fclose(fr);  
  fr = fopen("PARAGFS.TXT", "r");
  printf("\nI am working\nReading data from PARAGFS.TXT\n");
  outputr(fr);
  gstd = readsyva(fr);
  fclose(fr);  
  outputrw(fr2, fw);
  
  /* Reading input file INPUTGFS01A.TXT and writing the data from this file 
     to the results file */
  printf("Reading data from INPUTGFS01A.TXT\n");   
  n2 = 1;
  n3 = 1;
  zz++;
  inp00(fw,n2,n3,zz);
  fr = fopen("INPUTGFS01A.TXT", "r");
  fscanf(fr, "%f", &u);
  outputrw(fr2, fw);
  fprintf(fw, "%7.2f", u);
  fscanf(fr, "%d", &RRA);
  outputrw(fr2, fw);
  fprintf(fw, "%4d", RRA);
  outputrw(fr2, fw);
  for (i = 1; i <= RRA; i++) {
    fscanf(fr, "%lf", &l4RA[i]);  
    if (i < RRA) fprintf(fw, "   %8d                 %12.5f\n", i,l4RA[i]);
    if (i == RRA) fprintf(fw, "   %8d                 %12.5f", i,l4RA[i]);   
  } 
  outputrw(fr2, fw);
  fscanf(fr, "%d", &LLA);
  outputrw(fr2, fw);
  fprintf(fw, "%4d", LLA);
  outputrw(fr2, fw);
  for (i = 1; i <= LLA; i++) {
    fscanf(fr, "%lf", &l4EA[i]);  
    if (i < LLA) fprintf(fw, "   %8d                 %12.5f\n", i,l4EA[i]);
    if (i == LLA) fprintf(fw, "   %8d                 %12.5f", i,l4EA[i]);   
  } 
  outputrw(fr2, fw);
  for (i = 1; i < NT; i++) {
    fscanf(fr, "%f", &evpA[i][1]); 
    outputrw(fr2, fw);
    if (fabs(evpA[i][1]) < 0.000001) fprintf(fw, " not calculated");
      else  fprintf(fw, "%9.3f", evpA[i][1]);
  }
  fscanf(fr, "%f", &ageR11A);
  outputrw(fr2, fw);
  fprintf(fw, "%8.1f", ageR11A);
  fscanf(fr, "%f", &ageE1A);
  outputrw(fr2, fw);
  fprintf(fw, "%8.1f\n\n", ageE1A);
  fclose(fr);

  if (prods > 1) {
    /* Reading input file INPUTGFS01B.TXT and writing the data from this file 
       to the results file */
    printf("Reading data from INPUTGFS01B.TXT\n");   
    n2 = 1;
    n3 = 2;
    zz++;
    inp00(fw,n2,n3,zz);
    fr = fopen("INPUTGFS01B.TXT", "r");
    fscanf(fr, "%d", &RRB);
    outputrw(fr2, fw);
    fprintf(fw, "%4d", RRB);
    outputrw(fr2, fw);
    for (i = 1; i <= RRB; i++) {
      fscanf(fr, "%lf", &l4RB[i]);  
      if (i < RRB) fprintf(fw, "   %8d                 %12.5f\n", i,l4RB[i]);
      if (i == RRB) fprintf(fw, "   %8d                 %12.5f", i,l4RB[i]);   
    } 
    outputrw(fr2, fw);
    fscanf(fr, "%d", &LLB);
    outputrw(fr2, fw);
    fprintf(fw, "%4d", LLB);
    outputrw(fr2, fw);
    for (i = 1; i <= LLB; i++) {
      fscanf(fr, "%lf", &l4EB[i]);  
      if (i < LLB) fprintf(fw, "   %8d                 %12.5f\n", i,l4EB[i]);
      if (i == LLB) fprintf(fw, "   %8d                 %12.5f", i,l4EB[i]);   
    } 
    outputrw(fr2, fw);
    for (i = 1; i < NT; i++) {
      fscanf(fr, "%f", &evpB[i][1]); 
      outputrw(fr2, fw);
      if (fabs(evpB[i][1]) < 0.000001) fprintf(fw, " not calculated");
	else  fprintf(fw, "%9.3f", evpB[i][1]);
    }
    for (i = 1; i < NT; i++) {
      fscanf(fr, "%f", &evpB[i][2]); 
      outputrw(fr2, fw);
      if (fabs(evpB[i][2]) < 0.000001) fprintf(fw, " not calculated");
	else  fprintf(fw, "%9.3f", evpB[i][2]);
    }
    fscanf(fr, "%f", &ageR11B);
    outputrw(fr2, fw);
    fprintf(fw, "%8.1f", ageR11B);
    fscanf(fr, "%f", &ageR12A);
    outputrw(fr2, fw);
    fprintf(fw, "%8.1f", ageR12A);
    fscanf(fr, "%f", &ageE1B);
    outputrw(fr2, fw);
    fprintf(fw, "%8.1f", ageE1B);
    outputrw(fr2, fw);
    for (i = 1; i <= LLB; i++) {
      fscanf(fr, "%f", &pcrossE[i]);  
      if (i < LLB) fprintf(fw, "   %8d            %12.5f\n", i,pcrossE[i]);
      if (i == LLB) fprintf(fw, "   %8d            %12.5f", i,pcrossE[i]);   
    } 
    outputrw(fr2, fw);
    fclose(fr);
  }
  else for (i = 1; i <= 81; i++) outputr(fr2); 

  /* Calculation of the number of age classes */
  acsA = (int)(ageR11A/365)+RRA;
  acdA = (int)(ageE1A/365)+LLA;
  if (prods == 1) acm = acsA + acdA + 1;
  if (prods == 2) {
    acsB = (int)(ageR11B/365)+RRB;
    acdB = (int)(ageE1B/365)+LLB;
    acsAB = (int)(ageR12A/365)+RRB;
    acm = acsA + acdA + acsAB + acsB + acdB + 3;
  }
 
  /* Reading input file INPUTGFS02.TXT and writing the data from this file 
     to the results file. This file is read only for production system 1 with
     breed A. */
  if (prods == 1) {
    printf("Reading data from INPUTGFS02.TXT\n");   
    n2 = 2;
    n3 = 0;
    zz++;
    inp00(fw,n2,n3,zz);
    fr = fopen("INPUTGFS02.TXT", "r");
    readcom(fr);
    n_sac = readint(fw,fr);
    l_inv = readint(fw,fr);
    pgssA = readint(fw,fr);
    pgdsA = readint(fw,fr);
    if (pgssA == 1) {
      for (i = 1; i <= acsA; i++) fscanf(fr, "%f", &P[1][i]);
      wrides(fr,fw);
      for (i = 1; i <= acsA; i++) fprintf(fw, "%6.3f", P[1][i]);
    }
    else readdes(fr);
    if (pgdsA == 1) {
      for (i = 1; i <= acdA; i++) fscanf(fr, "%f", &P[1][acsA+i]);
      wrides(fr,fw);
      for (i = 1; i <= acdA; i++) fprintf(fw, "%6.3f", P[1][acsA+i]);
    }
    else readdes(fr);
    fclose(fr);
    /* Check for n_sac */
    if (n_sac < 1 || n_sac > acm-1) {
      nonsense02(acm,prods);
      goto END;
    }	
  }

  /* Reading input file INPUTGFS03.TXT and writing the data from this file 
     to the results file. This file is read only for production system 2 with
     breeds A and B and terminal crossing. */
  if (prods == 2) {
    printf("Reading data from INPUTGFS03.TXT\n");   
    n2 = 3;
    n3 = 0;
    zz++;
    inp00(fw,n2,n3,zz);
    fr = fopen("INPUTGFS03.TXT", "r");
    readcom(fr);
    pdA = readfloat(fw,fr);
    pdB = readfloat(fw,fr);
    n_sac = readint(fw,fr);
    l_inv = readint(fw,fr);
    pgssA = readint(fw,fr);
    pgdsA = readint(fw,fr);
    pgssB = readint(fw,fr);
    pgdsB = readint(fw,fr);
    if (pgssA == 1) {
      for (i = 1; i <= acsA; i++) fscanf(fr, "%f", &P[1][i]);
      wrides(fr,fw);
      for (i = 1; i <= acsA; i++) fprintf(fw, "%6.3f", P[1][i]);
    }
    else readdes(fr);
    if (pgdsA == 1) {
      for (i = 1; i <= acdA; i++) fscanf(fr, "%f", &P[1][acsA+i]);
      wrides(fr,fw);
      for (i = 1; i <= acdA; i++) fprintf(fw, "%6.3f", P[1][acsA+i]);
    }
    else readdes(fr); 
    if (pgssB == 1) {
      for (i = 1; i <= acsB; i++) 
        fscanf(fr, "%f", &P[acsA+acdA+acsAB+1][acsA+acdA+acsAB+i]);
      wrides(fr,fw);
      for (i = 1; i <= acsB; i++) 
        fprintf(fw, "%6.3f", P[acsA+acdA+acsAB+1][acsA+acdA+acsAB+i]);
    }
    else readdes(fr);
    if (pgdsB == 1) {
      for (i = 1; i <= acdB; i++) 
        fscanf(fr, "%f", &P[acsA+acdA+acsAB+1][acsA+acdA+acsAB+acsB+i]);
      wrides(fr,fw);
      for (i = 1; i <= acdB; i++) 
        fprintf(fw, "%6.3f", P[acsA+acdA+acsAB+1][acsA+acdA+acsAB+acsB+i]);	
    }
    else readdes(fr);
    fclose(fr);
    /* Check for n_sac */
    if (n_sac < 1 || n_sac > acm-3) {
      nonsense02(acm,prods);
      goto END;
    }  
  }

  /* Construction of matrix P for production system 1 */
  if (prods == 1) {
    /* Block 11 */
    if (pgssA == 2) for (i = 1; i <= RRA; i++) P[1][acsA-RRA+i] = 0.5 * l4RA[i];
    for (i = 1; i < acsA; i++) P[i+1][i] = 1;
    /* Block 12 */
    if (pgdsA == 2) 
      for (i = 1; i <= LLA; i++) P[1][acsA+acdA-LLA+i] = 0.5 * l4EA[i];
    /* Block 21 */
    for (i = 1; i <= RRA; i++) P[acsA+1][acsA-RRA+i] = 0.5 * l4RA[i];
    /* Block 22 */
    for (i = 1; i <= LLA; i++) P[acsA+1][acsA+acdA-LLA+i] = 0.5 * l4EA[i];
    for (i = 1; i < acdA; i++) P[acsA+i+1][acsA+i] = 1;
    /* Block 31 */
    for (i = 1; i <= RRA; i++) P[acm][acsA-RRA+i] = 0.5 * l4RA[i];
    /* Block 32 */
    for (i = 1; i <= LLA; i++) P[acm][acsA+acdA-LLA+i] = 0.5 * l4EA[i];
    /* Check for matrix P. All rows must sum to 1 */
    for (i = 1; i <= acm; i++) {
      a2 = 0;
      for (j = 1; j <= acm; j++) a2 += P[i][j];
      if (a2 < 0.99999 || a2 > 1.00001) {
        nonsense01(i,prods);
	goto END;
      }	
    }
  }

  if (prods == 1) {
    /* Calculation of vectors m_gft and m_sum for production system 1 */
    m_gf[n_sac] = 1;
    for (i1 = 1; i1 <= l_inv; i1++) { 
      for (i = 1; i <= acm; i++) {
	for (j = 1; j <= acm; j++) 
          m_gft[i] += P[i][j] * m_gf[j];  
      }
      for (i = 1; i <= acm; i++) { 
	m_gf[i] = m_gft[i]; 
	m_sum[i] += m_gf[i]/pow(u+1,i1); 
	if (i1 < l_inv) m_gft[i] = 0; 
      }
    } 
    /* Calculation of h_d and h_m for production system 1 */
    h_d[acm] = 1;
    for (i = 1; i <= LLA; i++) h_m[acsA+acdA-LLA+i] = l4EA[i];
    /* Calculation of the economic weights for production system 1 */
    a2 = a3 = 0;
    for (i = 1; i <= acm; i++) {
      a2 += h_d[i] * m_sum[i];
      a3 += h_m[i] * m_sum[i];
    }
    for (i = 1; i < NT; i++) {
      if (fabs(evpA[i][1]) < 0.000001) {
	ew_d[i] = 0.000000;
	ew_m[i] = 0.000000;
      }  
      else {
	if (tt[i] == 0 || tt[i] == 2) ew_d[i] = evpA[i][1] * a2;
	if (tt[i] == 1 || tt[i] == 2) ew_m[i] = evpA[i][1] * a3;
	if (tt[i] == 0) ew_m[i] = 0.000000;
	if (tt[i] == 1) ew_d[i] = 0.000000;	
      }  
    } 
  }
  
  /* Construction of matrix P for production system 2 */
  if (prods == 2) {
    /* Block 11 */
    if (pgssA == 2) for (i = 1; i <= RRA; i++) P[1][acsA-RRA+i] = 0.5 * l4RA[i]; 
    for (i = 1; i < acsA; i++) P[i+1][i] = 1;
    /* Block 12 */
    if (pgdsA == 2) 
      for (i = 1; i <= LLA; i++) P[1][acsA+acdA-LLA+i] = 0.5 * l4EA[i];
    /* Block 21 */
    for (i = 1; i <= RRA; i++) P[acsA+1][acsA-RRA+i] = 0.5 * l4RA[i];
    /* Block 22 */
    for (i = 1; i <= LLA; i++) P[acsA+1][acsA+acdA-LLA+i] = 0.5 * l4EA[i];
    for (i = 1; i < acdA; i++) P[acsA+i+1][acsA+i] = 1;
    /* Block 31 */
    for (i = 1; i <= acsA; i++) P[acsA+acdA+1][i] = P[1][i]; 
    /* Block 32 */
    for (i = 1; i <= acdA; i++) P[acsA+acdA+1][acsA+i] = P[1][acsA+i]; 
    /* Block 33 */
    for (i = 1; i < acsAB; i++) P[acsA+acdA+i+1][acsA+acdA+i] = 1;
    /* Block 44 */
    if (pgssB == 2) 
      for (i = 1; i <= RRB; i++) 
        P[acsA+acdA+acsAB+1][acsA+acdA+acsAB+acsB-RRB+i] = 0.5 * l4RB[i]; 
    for (i = 1; i < acsB; i++) P[acsA+acdA+acsAB+i+1][acsA+acdA+acsAB+i] = 1;
    /* Block 45 */
    if (pgdsB == 2) 
      for (i = 1; i <= LLB; i++) 
        P[acsA+acdA+acsAB+1][acsA+acdA+acsAB+acsB+acdB-LLB+i] 
	  = 0.5 * l4EB[i];
    /* Block 54 */
    for (i = 1; i <= RRB; i++) 
      P[acsA+acdA+acsAB+acsB+1][acsA+acdA+acsAB+acsB-RRB+i] 
        = 0.5 * l4RB[i];    
    /* Block 55 */
    for (i = 1; i <= LLB; i++) 
      P[acsA+acdA+acsAB+acsB+1][acsA+acdA+acsAB+acsB+acdB-LLB+i]
        = 0.5 * l4EB[i];
    for (i = 1; i < acdB; i++) 
      P[acsA+acdA+acsAB+acsB+i+1][acsA+acdA+acsAB+acsB+i] = 1;
    /* Block 61 */
    for (i = 1; i <= RRA; i++) 
      P[acm-2][acsA-RRA+i] = 0.5 * l4RA[i];    
    /* Block 62 */
    for (i = 1; i <= LLA; i++) P[acm-2][acsA+acdA-LLA+i] = 0.5 * l4EA[i];
    /* Block 74 */
    for (i = 1; i <= RRB; i++) 
      P[acm-1][acsA+acdA+acsAB+acsB-RRB+i] = 0.5 * l4RB[i];    
    /* Block 75 */
    a2 = 0;
    for (i = 1; i <= LLB; i++) a2 += l4EB[i] * (1 - pcrossE[i-1]);
    for (i = 1; i <= LLB; i++) 
      P[acm-1][acsA+acdA+acsAB+acsB+acdB-LLB+i]
        = 0.5 * l4EB[i] * (1 - pcrossE[i-1]) / a2;
    /* Block 83 */
    for (i = 1; i <= RRB; i++) 
      P[acm][acsA+acdA+acsAB-RRB+i] = 0.5 * l4RB[i];    
    /* Block 85 */
    a2 = 0;
    for (i = 1; i <= LLB; i++) a2 += l4EB[i] * pcrossE[i-1];
    for (i = 1; i <= LLB; i++) 
      P[acm][acsA+acdA+acsAB+acsB+acdB-LLB+i] 
        = 0.5 * l4EB[i] * pcrossE[i-1] / a2;
    /* Check for matrix P. All rows must sum to 1 */
    for (i = 1; i <= acm; i++) {
      a2 = 0;
      for (j = 1; j <= acm; j++) a2 += P[i][j];
      if (a2 < 0.99999 || a2 > 1.00001) {
        nonsense01(i,prods);
	goto END;
      }	
    }
  }
  if (prods == 2) {
    /* Calculation of vectors m_gft and m_sum for production system 2 */
    m_gf[n_sac] = 1;
    for (i1 = 1; i1 <= l_inv; i1++) { 
      for (i = 1; i <= acm; i++) {
	for (j = 1; j <= acm; j++) 
          m_gft[i] += P[i][j] * m_gf[j];  
      }
      for (i = 1; i <= acm; i++) { 
	m_gf[i] = m_gft[i]; 
	m_sum[i] += m_gf[i]/pow(u+1,i1); 
	if (i1 < l_inv) m_gft[i] = 0; 
      }
    } 
    /* Calculation of h_d and h_m for production system 2 */
    h_d[acm-2] = pdA;
    h_d[acm-1] = h_d[acm] = pdB;
    for (i = 1; i <= LLA; i++) h_m[acsA+acdA-LLA+i] = l4EA[i] * pdA;
    for (i = 1; i <= LLB; i++) 
      h_m[acm-3-LLB+i] = l4EB[i] * pdB;
    /* Calculation of he_d, he_m, ew_d and ew_m for production system 2 */
    for (i = 1; i < NT; i++) {
      he_d[i][acm-2] = h_d[acm-2] * evpA[i][1];
      he_d[i][acm-1] = h_d[acm-1] * evpB[i][1];
      he_d[i][acm] = h_d[acm] * evpB[i][2];    
      for (j = acsA + 1; j <= acsA + acdA; j++) 
        he_m[i][j] = h_m[j] * evpA[i][1];
      for (j = acm - acdB - 2; j <= acm - 3; j++)
        he_m[i][j] = h_m[j] * (evpB[i][1] + evpB[i][2]);
      ew_d[i] = ew_m[i] = 0.000000;
      for (j = 1; j <= acm; j++) {
        if (tt[i] == 0 || tt[i] == 2) ew_d[i] += he_d[i][j] * m_sum[j];
	if (tt[i] == 1 || tt[i] == 2) ew_m[i] += he_m[i][j] * m_sum[j];
      }	
    }
  } 

  /* Reading input file INPUTGFS04.TXT and writing the data from this file 
     to the results file. This file is read only if the genetic standard deviationsfor production system 2 with
     are read. */
  if (gstd == 2) {
    printf("Reading data from INPUTGFS04.TXT\n"); 
    fprintf(fw, "\n\n"); 
    n2 = 4;
    n3 = 0;
    zz++;
    inp00(fw,n2,n3,zz);
    fr = fopen("INPUTGFS04.TXT", "r");
    readcom(fr);
    for (i = 1; i < NT; i++) {
      if (fabs(ew_d[i]) < 0.000001 && fabs(ew_m[i]) < 0.000001) readdes(fr);
      else {
	if (tt[i] == 2) {
	  fscanf(fr, "%f", &gstd_d[i]); 
	  fscanf(fr, "%f", &gstd_m[i]); 
	  wrides(fr,fw); 
	  fprintf(fw, "%12.4f %12.4f", gstd_d[i],gstd_m[i]);
	  }
 	if (tt[i] == 0) {
	  fscanf(fr, "%f", &gstd_d[i]);
	  wrides(fr,fw); 
	  fprintf(fw, "%12.4f", gstd_d[i]);
	} 
	if (tt[i] == 1) {
	  fscanf(fr, "%f", &gstd_m[i]); 
	  wrides(fr,fw); 
          fprintf(fw, "%12.4f", gstd_m[i]);
	}   
      }  
    } 
  }
  
  /* Reading input file INPUTGFS05.TXT and writing the data from this file 
     to the results file. This file is read only if the genetic standard deviationsfor production system 2 with
     are read. */
  if (gstd == 2) {
    printf("Reading data from INPUTGFS05.TXT\n");  
    fprintf(fw, "\n\n");
    n2 = 5;
    n3 = 0;
    zz++;
    inp00(fw,n2,n3,zz);
    fr = fopen("INPUTGFS05.TXT", "r");
    readcom(fr);
    for (i = 1; i < 7; i++) 
      flag[i] = readint(fw,fr);
  }
  
  /* Selection bewtween two alternative trait definitions for several traits */
  if (flag[1] == 2) delta_d[3] = delta_m[3] = 0;
  if (flag[1] == 3) delta_d[2] = delta_m[2] = 0;
  if (flag[2] == 4) delta_d[5] = 0;
  if (flag[2] == 5) delta_d[4] = 0;
  if (flag[3] == 26) delta_m[27] = 0;
  if (flag[3] == 27) delta_m[26] = 0;
  if (flag[4] == 28) delta_m[29] = 0;
  if (flag[4] == 29) delta_m[28] = 0;
  if (flag[5] == 30) delta_m[31] = 0;
  if (flag[5] == 31) delta_m[30] = 0;
  if (flag[6] == 32) delta_m[33] = 0;
  if (flag[6] == 33) delta_m[32] = 0;
  
  /* Calculation of economic weights per genetic standard deviation and of relative
     economic weights */
  if (gstd == 2) {
    ewsum_d = ewsum_m = 0;
    for (i = 1; i < NT; i++) {
      if (fabs(ew_d[i]) > 0.000001 || fabs(ew_m[i]) > 0.000001) {
	if (tt[i] == 2 || tt[i] == 0) {
	  ewst_d[i] = ew_d[i] * gstd_d[i];
	  if (delta_d[i] == 1) ewsum_d += fabs(ewst_d[i]);  
	}  
	if (tt[i] == 2 || tt[i] == 1) {
	  ewst_m[i] = ew_m[i] * gstd_m[i];  
	  if (delta_m[i] == 1) ewsum_m += fabs(ewst_m[i]);
	}  
      }  
    }
    ewsum = ewsum_d + ewsum_m;
    for (i = 1; i < NT; i++) {
      if (fabs(ew_d[i]) > 0.000001 || fabs(ew_m[i]) > 0.000001) {
        if (delta_d[i] == 1) { 
	  ewr_dd[i] = 100 * fabs(ewst_d[i]) / ewsum_d;
	  ewr_da[i] = 100 * fabs(ewst_d[i]) / ewsum;
	}  
        if (delta_m[i] == 1) { 
	  ewr_mm[i] = 100 * fabs(ewst_m[i]) / ewsum_m;
	  ewr_ma[i] = 100 * fabs(ewst_m[i]) / ewsum;
	}  
      }
    }  
  }     
       
Printabc: /* Printing variable values to CHECKGFS for checking the calulation */
  printf("Writing results to file CHECKGFS\n");
  fw2 = fopen("CHECKGFS", "w");
  fprintf(fw2, "a2 = %8.3f\n", a2);
  fprintf(fw2, "a3 = %8.3f\n", a3);
  fprintf(fw2, "ac = %4d\n", ac);
  fprintf(fw2, "acdA = %4d\n", acdA);
  if (prods > 1) fprintf(fw2, "acdB = %4d\n", acdB);
  fprintf(fw2, "ACM = %4d\n", ACM);
  fprintf(fw2, "acm = %4d\n", acm);
  fprintf(fw2, "acsA = %4d\n", acsA);
  if (prods > 1) fprintf(fw2, "acsAB = %4d\n", acsAB);
  if (prods > 1) fprintf(fw2, "acsB = %4d\n", acsB);
  fprintf(fw2, "ageE1A = %8.1f\n", ageE1A);
  if (prods > 1) fprintf(fw2, "ageE1B = %8.1f\n", ageE1B);
  fprintf(fw2, "ageR11A = %8.1f\n", ageR11A);
  if (prods > 1) fprintf(fw2, "ageR11B = %8.1f\n", ageR11B);
  if (prods > 1) fprintf(fw2, "ageR12A = %8.1f\n", ageR12A);
  fprintf(fw2, "CB = %4d\n", CB);
  fprintf(fw2, "\n");
  if (gstd == 2) {
    for (i = 1; i < NT; i++) 
      fprintf(fw2, "delta_d[%d] = %3d\n",i, delta_d[i]);
    fprintf(fw2, "\n");  
    for (i = 1; i < NT; i++) 
      fprintf(fw2, "delta_m[%d] = %3d\n",i, delta_m[i]);
    fprintf(fw2, "\n");  
  } 
  fprintf(fw2, "errmess = %4d\n", errmess);  
  fprintf(fw2, "\n");
  for (i = 1; i < NT; i++) {
    if (fabs(evpA[i][1]) < 0.000001) fprintf(fw2, "evpA[%d][1] = not calculated\n", i);
    else fprintf(fw2, "evpA[%d][1] = %12.4f\n",i, evpA[i][1]);
  }  
  fprintf(fw2, "\n");
  if (prods > 1) {  
    for (i = 1; i < NT; i++) {
      if (fabs(evpB[i][1]) < 0.000001) fprintf(fw2, "evpB[%d][1] = not calculated\n", i);
      else fprintf(fw2, "evpB[%d][1] = %12.4f\n",i, evpB[i][1]);
    }  
    fprintf(fw2, "\n");
    for (i = 1; i < NT; i++) {
      if (fabs(evpB[i][2]) < 0.000001) fprintf(fw2, "evpB[%d][2] = not calculated\n", i);
      else fprintf(fw2, "evpB[%d][2] = %12.4f\n",i, evpB[i][2]);
    }  
    fprintf(fw2, "\n");
  }  
  for (i = 1; i < NT; i++) {
    if (fabs(ew_d[i]) < 0.000001) fprintf(fw2, "ew_d[%d] = not calculated\n", i);
    else fprintf(fw2, "ew_d[%d] = %12.4f\n",i, ew_d[i]);
  }  
  fprintf(fw2, "\n");  
  for (i = 1; i < NT; i++) {
    if (fabs(ew_m[i]) < 0.000001) fprintf(fw2, "ew_m[%d] = not calculated\n", i);
    else fprintf(fw2, "ew_m[%d] = %12.4f\n",i, ew_m[i]);
  }  
  fprintf(fw2, "\n");  
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ewr_da[i]) > 0.000001) 
	fprintf(fw2, "ewr_da[%d] = %12.4f\n",i, ewr_da[i]);
    }  
    fprintf(fw2, "\n");  
  }
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ewr_dd[i]) > 0.000001) 
	fprintf(fw2, "ewr_dd[%d] = %12.4f\n",i, ewr_dd[i]);
    }  
    fprintf(fw2, "\n");  
  }
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ewr_ma[i]) > 0.000001) 
	fprintf(fw2, "ewr_ma[%d] = %12.4f\n",i, ewr_ma[i]);
    }  
    fprintf(fw2, "\n");  
  }
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ewr_mm[i]) > 0.000001) 
	fprintf(fw2, "ewr_mm[%d] = %12.4f\n",i, ewr_mm[i]);
    }  
    fprintf(fw2, "\n");  
  }
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ew_d[i]) > 0.000001) 
	fprintf(fw2, "ewst_d[%d] = %12.4f\n",i, ewst_d[i]);
    }  
    fprintf(fw2, "\n");  
  }
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ew_m[i]) > 0.000001) 
	fprintf(fw2, "ewst_m[%d] = %12.4f\n",i, ewst_m[i]);
    }  
    fprintf(fw2, "\n");  
  }
  if (gstd == 2) fprintf(fw2, "ewsum = %10.3f\n", ewsum);
  if (gstd == 2) fprintf(fw2, "ewsum_d = %10.3f\n", ewsum_d);
  if (gstd == 2) fprintf(fw2, "ewsum_m = %10.3f\n", ewsum_m);
  if (gstd == 2) fprintf(fw2, "\n");
  if (gstd == 2) {
    for (i = 1; i < 7; i++) 
      fprintf(fw2, "flag[%d] = %3d\n",i, flag[i]);
    fprintf(fw2, "\n");  
  }
  fprintf(fw2, "gstd = %4d\n", gstd);
  fprintf(fw2, "\n");  
  if (gstd == 2) {
    for (i = 1; i < NT; i++) {
      if (fabs(ew_d[i]) > 0.000001) 
        fprintf(fw2, "gstd_d[%d] = %12.4f\n",i, gstd_d[i]);
    }  
    fprintf(fw2, "\n");  
    for (i = 1; i < NT; i++) {
      if (fabs(ew_m[i]) > 0.000001) 
        fprintf(fw2, "gstd_m[%d] = %12.4f\n",i, gstd_m[i]);
    }  
    fprintf(fw2, "\n");  
  }
  for (i = 1; i <= acm; i++) 
    if (fabs(h_d[i]) > 0.000001) fprintf(fw2, "h_d[%d] = %8.5f\n",i, h_d[i]); 
  fprintf(fw2, "\n");  
  for (i = 1; i <= acm; i++) 
    if (fabs(h_m[i]) > 0.000001) fprintf(fw2, "h_m[%d] = %8.5f\n",i, h_m[i]); 
  fprintf(fw2, "\n");  
  if (prods > 1) {
    for (i = 1; i < NT; i++) {
      for (j = 1; j <= acm; j++)
	if (fabs(he_d[i][j]) > 0.000001) 
          fprintf(fw2, "he_d[%d][%d] = %12.4f\n", i, j, he_d[i][j]);
    }  
    fprintf(fw2, "\n");
    for (i = 1; i < NT; i++) {
      for (j = 1; j <= acm; j++)
	if (fabs(he_m[i][j]) > 0.000001) 
          fprintf(fw2, "he_m[%d][%d] = %12.4f\n", i, j, he_m[i][j]);
    }  
    fprintf(fw2, "\n");
  }  
  fprintf(fw2, "L = %4d\n", L);  
  fprintf(fw2, "l_inv = %4d\n", l_inv);     
  fprintf(fw2, "\n");
  for (i = 1; i <= LLA; i++) 
    fprintf(fw2, "l4EA[%d] = %8.5f\n",i, l4EA[i]); 
  fprintf(fw2, "\n"); 
  if (prods > 1) {
    for (i = 1; i <= LLB; i++) 
      fprintf(fw2, "l4EB[%d] = %8.5f\n",i, l4EB[i]); 
    fprintf(fw2, "\n");
  } 
  for (i = 1; i <= RRA; i++) 
    fprintf(fw2, "l4RA[%d] = %8.5f\n",i, l4RA[i]); 
  fprintf(fw2, "\n");  
  if (prods > 1) {
    for (i = 1; i <= RRB; i++) 
      fprintf(fw2, "l4RB[%d] = %8.5f\n",i, l4RB[i]); 
    fprintf(fw2, "\n");  
  }
  fprintf(fw2, "LLA = %4d\n", LLA); 
  if (prods > 1) fprintf(fw2, "LLB = %4d\n", LLB);  
  fprintf(fw2, "\n"); 
  for (i = 1; i <= acm; i++) 
    if (fabs(m_gf[i]) > 0.000001) fprintf(fw2, "m_gf[%d] = %8.5f\n",i, m_gf[i]); 
  fprintf(fw2, "\n");  
  for (i = 1; i <= acm; i++) 
    if (fabs(m_gft[i]) > 0.000001) fprintf(fw2, "m_gft[%d] = %8.5f\n",i, m_gft[i]); 
  fprintf(fw2, "\n");  
  for (i = 1; i <= acm; i++) 
    if (fabs(m_sum[i]) > 0.000001) fprintf(fw2, "m_sum[%d] = %8.5f\n",i, m_sum[i]); 
  fprintf(fw2, "\n");  
  fprintf(fw2, "n_sac = %4d\n", n_sac);  
  fprintf(fw2, "n2 = %4d\n", n2);  
  fprintf(fw2, "n3 = %4d\n", n3);  
  fprintf(fw2, "NT = %4d\n", NT);
  fprintf(fw2, "\n");  
  for (i=1; i<=acm; i++) {
    for (j=1; j<=acm; j++)     
      if (fabs(P[i][j])>.000001)
        fprintf(fw2, "P[%d][%d] = %12.6f\n", i, j, P[i][j]);
  }
  fprintf(fw2, "\n");
  if (prods > 1) {  
    for (i = 0; i < LLB; i++) 
      fprintf(fw2, "pcrossE[%d] = %8.3f\n",i, pcrossE[i]); 
    fprintf(fw2, "\n");
  }
  if (prods > 1) fprintf(fw2, "pdA = %8.3f\n", pdA);
  if (prods > 1) fprintf(fw2, "pdB = %8.3f\n", pdB); 
  fprintf(fw2, "pgdsA = %4d\n", pgdsA);
  if (prods > 1) fprintf(fw2, "pgdsB = %4d\n", pgdsB);
  fprintf(fw2, "pgssA = %4d\n", pgssA); 
  if (prods > 1) fprintf(fw2, "pgssB = %4d\n", pgssB); 
  fprintf(fw2, "prods = %4d\n", prods);
  fprintf(fw2, "R = %4d\n", R);
  fprintf(fw2, "RRA = %4d\n", RRA);   
  if (prods > 1) fprintf(fw2, "RRB = %4d\n", RRB);        
  fprintf(fw2, "\n");    
  for (i = 1; i < NT; i++) 
    fprintf(fw2, "tt[%d] = %3d\n",i, tt[i]);
  fprintf(fw2, "\n");    
  fprintf(fw2, "u = %7.2f\n", u);        
  fprintf(fw2, "zz = %4d\n", zz);  
  
  fclose(fw2);  
  
res: printf("Writing the results to the results file resgfps0%d.\n", prods);
  outputrw(fr2, fw);
  fprintf(fw, "%2d.\n", prods);
  /* 3.1 Cumulative number of discounted expressions in all sex and age groups at
         the end of the investment period (vector m_sum) */
  outputrw(fr2, fw);
  if (prods == 1) {
    if (n_sac <= acsA) {
      ac = n_sac;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
    if (n_sac > acsA) {
      ac = n_sac - acsA;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
  }
  else for (i = 1; i <= 2; i++) outputr(fr2);
  if (prods == 2) {
    if (n_sac <= acsA) {
      ac = n_sac;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
    if (n_sac > acsA && n_sac <= acsA + acdA) {
      ac = n_sac - acsA;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
    if (n_sac > acsA + acdA && n_sac <= acsA + acdA + acsAB) {
      ac = n_sac - acsA - acdA;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
    if (n_sac > acsA + acdA + acsAB && n_sac <= acsA + acdA + acsAB + acsB) {
      ac = n_sac - acsA - acdA - acsAB;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
    if (n_sac > acsA + acdA + acsAB + acsB && n_sac <= acm - 3) {
      ac = n_sac - acsA - acdA - acsAB - acsB;
      fprintf(fw, "%3d", ac);
      outputrw(fr2, fw);
    }
    else outputr(fr2);
  }
  else for (i = 1; i <= 5; i++) outputr(fr2);
  outputrw(fr2, fw);
  fprintf(fw, "%4d\n", l_inv);
  outputrw(fr2, fw);
  for (i = 1; i <= acsA; i++) 
    fprintf(fw, "          %3d                                     %8.3f\n",
      i, m_sum[i]); 
  outputrw(fr2, fw);
  for (i = 1; i <= acdA; i++) 
    fprintf(fw, "          %3d                                     %8.3f\n",
      i, m_sum[acsA+i]); 
  if (prods == 2) {
    outputrw(fr2, fw);
    for (i = 1; i <= acsAB; i++) 
      fprintf(fw, "          %3d                                     %8.3f\n",
	i, m_sum[acsA+acdA+i]); 
    outputrw(fr2, fw);
    for (i = 1; i <= acsB; i++) 
      fprintf(fw, "          %3d                                     %8.3f\n",
	i, m_sum[acsA+acdA+acsAB+i]); 
    outputrw(fr2, fw);
    for (i = 1; i <= acdB; i++) 
      fprintf(fw, "          %3d                                     %8.3f\n",
	i, m_sum[acsA+acdA+acsAB+acsB+i]); 
  }
  else for (i = 1; i <= 3; i++) outputr(fr2);
  outputrw(fr2, fw);
  if (prods == 1)
    fprintf(fw, "          %3d                                     %8.3f",
      1, m_sum[acm]); 
  if (prods == 2)
    fprintf(fw, "          %3d                                     %8.3f\n",
      1, m_sum[acm-2]); 
  if (prods == 2) {
    outputrw(fr2, fw);
    fprintf(fw, "          %3d                                     %8.3f\n",
      1, m_sum[acm-1]); 
    outputrw(fr2, fw);
    fprintf(fw, "          %3d                                     %8.3f",
      1, m_sum[acm]); 
  }
  else for (i = 1; i <= 2; i++) outputr(fr2);  
  /* 3.2 Economic weights */
  outputrw(fr2, fw);
  for (i = 1; i < NT; i++) {
    if (fabs(ew_d[i]) < 0.000001 && fabs(ew_m[i]) < 0.000001) outputr(fr2);
    else {
      outputrw(fr2, fw);
      if (tt[i] == 2) 
        fprintf(fw, "                                            %10.4f         %10.4f\n",
	   ew_d[i], ew_m[i]);
      if (tt[i] == 0) 
        fprintf(fw, "                                            %10.4f               -\n",
	   ew_d[i]);
      if (tt[i] == 1) 
        fprintf(fw, "                                                  -            %10.4f\n",
	   ew_m[i]);
    }
  }
  outputrw(fr2, fw);
  /* 3.3 Economic weights per genetic standard deviation */
  if (gstd == 2) {
    outputrw(fr2, fw);
    for (i = 1; i < NT; i++) {
      if (fabs(ew_d[i]) < 0.000001 && fabs(ew_m[i]) < 0.000001) outputr(fr2);
      else {
	outputrw(fr2, fw);
	if (tt[i] == 2) 
          fprintf(fw, "                                            %10.4f         %10.4f\n",
	     ewst_d[i], ewst_m[i]);
	if (tt[i] == 0) 
          fprintf(fw, "                                            %10.4f               -\n",
	     ewst_d[i]);
	if (tt[i] == 1) 
          fprintf(fw, "                                                  -            %10.4f\n",
	     ewst_m[i]);
      }
    }
    outputrw(fr2, fw);
    fprintf(fw, "                                            %10.4f         %10.4f\n",
      ewsum_d, ewsum_m);
    outputrw(fr2, fw);
    fprintf(fw, "                                            %10.4f         %10.4f\n",
      100 * ewsum_d / ewsum, 100 * ewsum_m / ewsum);
    outputrw(fr2, fw);  
  }
  /* 3.4 Relative economic weights */
  /* 3.4.1 Standardized economic weights for direct or maternal trait components as 
           percentage of the sum of absolute values of standardized economic values for 
	   direct or maternal trait components, respectively */
  if (gstd == 2) {
    outputrw(fr2, fw);
    for (i = 1; i < NT; i++) {
      if (fabs(ewr_dd[i]) < 0.000001 && fabs(ewr_mm[i]) < 0.000001) outputr(fr2);
      else {
	outputrw(fr2, fw);
	if (tt[i] == 2) 
          fprintf(fw, "                                            %10.3f         %10.3f\n",
	     ewr_dd[i], ewr_mm[i]);
	if (tt[i] == 0) 
          fprintf(fw, "                                            %10.3f               -\n",
	     ewr_dd[i]);
	if (tt[i] == 1) 
          fprintf(fw, "                                                  -            %10.3f\n",
	     ewr_mm[i]);
      }
    }
    outputrw(fr2, fw);
  }  
  /* 3.4.2 Standardized economic weights for direct or maternal trait components as 
           percentage of the sum of absolute values of standardized economic values 
           for both direct or maternal trait components */
  if (gstd == 2) {
    outputrw(fr2, fw);
    for (i = 1; i < NT; i++) {
      if (fabs(ewr_da[i]) < 0.000001 && fabs(ewr_ma[i]) < 0.000001) outputr(fr2);
      else {
	outputrw(fr2, fw);
	if (tt[i] == 2) 
          fprintf(fw, "                                            %10.3f         %10.3f\n",
	     ewr_da[i], ewr_ma[i]);
	if (tt[i] == 0) 
          fprintf(fw, "                                            %10.3f               -\n",
	     ewr_da[i]);
	if (tt[i] == 1) 
          fprintf(fw, "                                                  -            %10.3f\n",
	     ewr_ma[i]);
      }
    }
    outputrw(fr2, fw);
      fprintf(fw, "                                         %10.3f         %10.3f",
        100 * ewsum_d / ewsum, 100 * ewsum_m / ewsum);
    outputrw(fr2, fw);
  }  
  fclose(fr2);
  fclose(fw);
  
  printf("\nThank you for using the program GFSH.\n");
  if (prods == 1) printf("The results are given in the file resgfps01.\n");
  if (prods == 2) printf("The results are given in the file resgfps02.\n");
  printf("The values of all variables are given in the file CHECKGFS.\n");
END:  kon = clock();
  printf("The program needed %6.2f seconds.\n", ((double) (kon - zac)) / ((double) CLOCKS_PER_SEC));
      
} /* End of the main program */

/* Function for reading data from file TEXTGFS_OUT.TXT and writing them to the output file*/
void outputrw(FILE *fr, FILE *fw)
{
  int a, a1, z;
    a = 0;
    do {z = getc(fr);
        if (z == '"') a++;
	if (z != '"') putc(z, fw); 
       } while (a < 2);
}
/* Function for reading data from file TEXTGFS_OUT.TXT without writing them to the output file*/
void outputr(FILE *fr)
{
  int a, a1, z;
    a = 0;
    do {z = getc(fr);
        if (z == '"') a++;	
       } while (a<2);
}
/* Function for reading system variables and writing them to output file */
void sysvar(FILE *fr, FILE *fw)
{
  int a, b, i, z;
  for (i = 1; i <= 1; i++)
  { fscanf(fr, "%d", &b);
    a = 0;
    do {z = getc(fr);
        if (z == '"') a++;
	if (z != '"') putc(z, fw); 
       } while (a<2);
    fprintf(fw, ":  %d", b);   
  }
  fprintf(fw, "\n");   
}

/* Function for reading system variables to the main program */
int readsyva(FILE *fr)
{
  int a, b, z;
  fscanf(fr, "%d", &b);
  a = 0;
  do {z = getc(fr);
      if (z=='"') a++;
  } while (a < 2);  
  return b;
}

/* Skipping comments in the input files */
void readcom(FILE *fr)
{
  int a, z;
  a = 0;
  do {z = getc(fr);
      if (z=='/') a++;
  } while (a < 2);  
}

/* Function for reading data in format float */
float readdat(FILE *fr)
{
  float pa;
  fscanf(fr, "%f", &pa);
  readdes(fr);
  return pa;
}

/* Function for reading data in format int */
int readdat1(FILE *fr)
{
  int pa;
  fscanf(fr, "%d", &pa);
  readdes(fr);
  return pa;
}

/* Function for reading data in format float and writing them to results */
float readfloat(FILE *fw, FILE *fr)
{
  int a, a1, z;
  float d2;
  fscanf(fr, "%f", &d2);
  a = 0;
  do {z = getc(fr);
      if (z == '"') a++;
      if (z != '"') putc(z, fw); 
     } while (a<2);
  a = 0;
  a1 = 0;    
  do {z = getc(fr);
      if (z == '"') a++;
      if (a == 1 && a1 == 0)
	{ fprintf(fw, "   (");
	  a1++;
	} 
      if (z != '"') putc(z, fw); 
     } while (a<2);
    fprintf(fw,"):%12.4f ", d2);   
  return d2;
}

/* Function for reading data in format int and writing them to results */
int readint(FILE *fw, FILE *fr)
{
  int a, a1, z, d2;
  fscanf(fr, "%d", &d2);
  a = 0;
  do {z = getc(fr);
      if (z == '"') a++;
      if (z != '"') putc(z, fw); 
     } while (a<2);
  a = 0;
  a1 = 0;    
  do {z = getc(fr);
      if (z == '"') a++;
      if (a == 1 && a1 == 0)
	{ fprintf(fw, "   (");
	  a1++;
	} 
      if (z != '"') putc(z, fw); 
     } while (a<2);
    fprintf(fw,"):  %d ", d2);   
  return d2;
}

/* Skipping the description and the unit of the input parameter */
void readdes(FILE *fr)
{
  int a, z;
  a = 0;
  do {z = getc(fr);
      if (z=='"') a++;
  } while (a < 4);  
}

/* Function for reading the head of a input file
   and writing it to the output file*/
void inp00(FILE *fw, int n2, int n3, int zz)
{
  if (zz<10) fprintf(fw, "2.%1d Data from file ", zz);
    else fprintf(fw, "2.%2d Data from file ", zz);
  if (n2<10 && n3==0) fprintf(fw, "INPUTGFS0%d.TXT\n", n2);
  if (n2>9 && n3==0) fprintf(fw, "INPUTGFS%d.TXT\n", n2);
  if (n2<10 && n3==1) fprintf(fw, "INPUTGFS0%dA.TXT\n", n2);
  if (n2<10 && n3==2) fprintf(fw, "INPUTGFS0%dB.TXT\n", n2);
  if (n2>9 && n3==1) fprintf(fw, "INPUTGFS%dA.TXT\n", n2);
  if (n2>9 && n3==2) fprintf(fw, "INPUTGFS%dB.TXT\n", n2);
  if (zz<10 && n3==0) fprintf(fw, "---------------------------------");
  if ((zz>=10 && n3==0) || (zz<10 && n3>0)) 
    fprintf(fw, "----------------------------------");
  if (zz>=10 && n3>0) fprintf(fw, "-----------------------------------");
}  

/* Function for reading system variables and writing them to output file */
void wrides(FILE *fr, FILE *fw)
{
int a, a1, z;
  fprintf(fw, "\n");
  a = 0;
  do {z = getc(fr);
      if (z == '"') a++;
      if (z != '"' && a > 0) putc(z, fw); 
     } while (a<2);
  a = 0;
  a1 = 0;    
  do {z = getc(fr);
      if (z == '"') a++;
      if (a == 1 && a1 == 0)
	{ fprintf(fw, "   (");
	  a1++;
	} 
      if (z != '"') putc(z, fw); 
     } while (a<2);
  fprintf(fw,"):");
}

/* The ith row in matrix P does not sum to 1 */
void nonsense01(int i, int prods)
{
  printf("\nError message:\n");
  printf("Something is going wrong with matrix P.\n"); 
  printf("  The row # %d does not sum to 1.\n",i);
  if (prods == 1)
    printf("  Check your data in input files INPUTGFS01A.TXT and INPUTGFS02.TXT. \n\n\n");
  if (prods == 2)  
    printf("  Check your data in input files INPUTGFS01B.TXT and INPUTGFS03.TXT. \n\n\n");
}
/* Wrong value for n_sac */
void nonsense02(int acm, int prods)
{
  printf("\n\n\nError message:\n");
  printf("The number of the sex and age group for which the gene flow will be calculated\n"); 
  if (prods == 1) printf("  is wrong. Its value must be between 1 and %d.\n", acm - 1);
  if (prods == 2) printf("  is wrong. Its value must be between 1 and %d.\n", acm - 3);
  if (prods == 1) printf("  Correct the value in INPUTGFS02.TXT and restart the program.");
  if (prods == 2) printf("  Correct the value in INPUTGFS03.TXT and restart the program.");
  printf("\n\n\n");
}
